@@ -1,4 +1,5 @@
 import json
+import os.path
 from typing import Dict, Any, Optional, List
 
 from requests_toolbelt import MultipartEncoder
@@ -159,6 +160,38 @@ class X_API:
         response = self._client.open_api_do("POST", "search/tag", json=post_data)
         return response.json()
 
+    def get_dataset_path(
+            self,
+            dataset_id: str,
+            path: Optional[str] = ""
+    ) -> str:
+        path_root = self.get_dataset_path_root(dataset_id)
+        prefix = os.path.join(path_root, path.strip("/")).rstrip("/")
+        file_path = os.path.dirname(prefix)
+        base_name = os.path.basename(prefix)
+
+        params: Dict[str, Any] = {"prefix": file_path}
+        response = self._client.open_api_do("GET", "path", dataset_id, params=params).json()
+
+        if base_name not in response["data"]:
+            raise ResourceNotExistError(resource="path", identification=path)
+
+        return prefix[len(dataset_id) + 1:]
+
+    def get_dataset_path_root(
+            self,
+            dataset_id: str
+    ) -> str:
+        response = self._client.open_api_do("GET", "path", dataset_id).json()
+
+        try:
+            path_root = os.path.dirname(response["data"][0])
+        except IndexError as error:
+            raise ResourceNotExistError(resource="dataset path root", identification=dataset_id) from error
+
+        return path_root
+
+
     def get_authorized_s3_config(
             self,
             name: Optional[str] = "",
