@@ -2,7 +2,7 @@ import json
 import logging
 import os.path
 import time
-from typing import List, Dict, Optional, Any, TYPE_CHECKING
+from typing import List, Optional, Any, TYPE_CHECKING
 
 from s3transfer.constants import GB
 from tqdm import tqdm
@@ -10,7 +10,7 @@ from urllib.parse import urlparse
 
 from metaloop.client.cloud_storage import CloudConfig, CloudClient, Item
 from metaloop.client.download_helper import DownloadHelper
-from metaloop.utils.file_helper import get_file_type, FileTypeZip
+from metaloop.utils.file_helper import *
 from metaloop.exception import ResourceNotExistError, InvalidParamsError, InternalServerError
 
 if TYPE_CHECKING:
@@ -222,6 +222,7 @@ class Dataset(DatasetMeta):
             file_path: str,
             import_type: Optional[str] = "none",
             storage_type: Optional[str] = "local",
+            append_path: Optional[str] = "",
             transcode: Optional[bool] = False
     ) -> None:
         """Upload and import data to dataset from local.
@@ -235,7 +236,8 @@ class Dataset(DatasetMeta):
                 1. "local": MetaLoop default local object storage;
                 2. "cos": Tencent Cloud Object Storage (COS).
 
-            transcode: whether converse format of data, only used for video.
+            append_path: The specified path to store data in the object storage (without bucket).
+            transcode: Whether converse format of data, only used for video.
 
         """
         if not file_path:
@@ -244,6 +246,9 @@ class Dataset(DatasetMeta):
         if storage_type not in {"local", "cos"}:
             raise InvalidParamsError(param_name="storage_type", param_value=storage_type)
 
+        if append_path:
+            append_path = self._x_api.get_dataset_path(self._id, append_path)
+
         print(f"upload file to server, please wait...")
 
         if storage_type == "cos":
@@ -251,17 +256,20 @@ class Dataset(DatasetMeta):
                 file_type = get_file_type(file_path)[1]
                 if file_type == FileTypeZip:
                     raise InvalidParamsError(message="Tencent COS uploading do not support compressed file")
-                upload_id = self._upload_file(file_path, storage_type)
+                upload_id = self._upload_file(file_path, storage_type, append_path)
             else:
-                upload_id = self._upload_files(file_path, storage_type, self._id)
+                upload_prefix = append_path if append_path else self._id
+                upload_id = self._upload_files(file_path, storage_type, upload_prefix)
             task_type = "cos"
 
         elif os.path.isfile(file_path) and os.path.getsize(file_path) > 1 * GB:
-            upload_id = self._upload_file(file_path, storage_type)
+            upload_id = self._upload_file(file_path, storage_type, append_path)
             task_type = "object_storage"
 
         elif os.path.isdir(file_path):
-            upload_id = self._upload_files(file_path, storage_type)
+            if append_path:
+                raise InvalidParamsError(message="Directory uploading do not support append_path, please try with zip")
+            upload_id = self._upload_files(file_path, storage_type, append_path)
             task_type = "object_storage"
 
         else:
@@ -279,6 +287,7 @@ class Dataset(DatasetMeta):
             ],
             "import_type": import_type,
             "task_type": task_type,
+            "append_path": append_path
         }
 
         self._x_api.post_import(self._id, post_data)
@@ -294,6 +303,7 @@ class Dataset(DatasetMeta):
             self,
             file_path: str,
             storage_type: Optional[str] = "local",
+            export_path: Optional[List[str]] = None
     ) -> None:
         """Export data to local file from dataset
 
@@ -304,6 +314,8 @@ class Dataset(DatasetMeta):
                 1. "local": MetaLoop default local object storage;
                 2. "external": Tencent Cloud Object Storage (COS).
 
+            export_path: Export data from the specific path[s].
+
         """
         if storage_type not in {"local", "external"}:
             raise InvalidParamsError(param_name="storage_type", param_value=storage_type)
@@ -317,6 +329,9 @@ class Dataset(DatasetMeta):
             "storage_type": storage_type,
             "task_type": "shared_links",
         }
+        if export_path:
+            export_path = self._x_api.get_dataset_path(self._id, export_path)
+            post_data["export_path"] = f"[{','.join(export_path)}]"
 
         self._x_api.post_export(self._id, post_data)
         print("export data start, please wait...")
@@ -462,13 +477,14 @@ class Dataset(DatasetMeta):
     def _upload_file(
             self,
             file_path: str,
-            storage_type: str
+            storage_type: str,
+            upload_prefix: Optional[str] = ""
     ) -> str:
         bucket = self._get_bucket(storage_type)
         s3_config = self._get_cloud_storage_config("", storage_type)
         if not bucket:
             bucket = s3_config.default_bucket
-        return CloudClient.upload_file(s3_config.identifier, bucket, file_path)
+        return CloudClient.upload_file(s3_config.identifier, bucket, file_path, upload_prefix)
 
     def _upload_files(
             self,
@@ -512,16 +528,15 @@ class Dataset(DatasetMeta):
             file_path: str,
             image_path: str
     ) -> (str, str, str):
-        index = url.strip("/").index("/")
-        key = url[index + 2:]
+        _, key = get_bucket_and_key(url)
 
-        if image_path:
-            file_name = os.path.join(file_path, image_path.strip("/"))
-        else:
-            file_name = os.path.join(file_path, url.strip("/"))
+        if not image_path:
+            image_path = get_user_origin_path(key)
 
-        if not file_path:
-            file_name = os.path.join("abaddon", file_name)
+        if not file_path or image_path.find("/") < 0:
+            image_path = os.path.join("abaddon", image_path)
+
+        file_name = os.path.join(file_path, image_path.strip("/"))
 
         if not os.path.exists(os.path.dirname(file_name)):
             os.makedirs(os.path.dirname(file_name), 0o0755)
@@ -598,8 +613,18 @@ class Dataset(DatasetMeta):
             url: str,
             file_path: str
     ) -> str:
-        index = url.strip("http://").index("/")
-        file_name = os.path.join(file_path, url[index + 8:])
+        if url.startswith("http://"):
+            file_name = url[url.index("http://") + 7:]
+        elif url.startswith("https://"):
+            file_name = url[url.index("https://") + 8:]
+        else:
+            file_name = url
+
+        index = file_name.find("/")
+        if index > -1:
+            file_name = os.path.join(file_path, file_name[index + 1:])
+        else:
+            file_name = os.path.join(file_path, file_name)
 
         if not os.path.exists(os.path.dirname(file_name)):
             os.makedirs(os.path.dirname(file_name), 0o0755)
