@@ -17,7 +17,7 @@ if __name__ == '__main__':
     )
 
     # import directory
-    dataset.import_data("import_test/sample_test")
+    dataset.import_data("import_test/sample_test.zip")
 
     # export data to local
     dataset.export_data("export_test")
